@@ -1,6 +1,7 @@
 #!/bin/sh
-OTHER_PACKAGES="lvm2 gdisk grub-pc"
+PACKAGES="lvm2 gdisk e2fsprogs"
 eval "$chrooted_functions"
+probe_target_optional_functions
 start_failsafe_mode
 # in the chroot commands should use /tmp for temporary files
 export TMPDIR=/tmp
@@ -27,11 +28,12 @@ failsafe mount -t proc none /proc
 failsafe_mount_sys_and_dev
 export DEBIAN_FRONTEND=noninteractive LANG=C
 
-# if update-grub is called as part of the package installation
-# it should properly find our virtual device.
-# (we will properly install the bootloader on the final device
-# anyway, this is only useful to avoid warnings)
-update_grup_device_map $loop_device
+optional_target_prepare_rootfs draft inside
+
+if $target_custom_packages_exists
+then
+    PACKAGES="$PACKAGES $(target_custom_packages)"
+fi
 
 # install missing packages
 echo -n "I: draft image - updating package manager database... "
@@ -41,17 +43,7 @@ echo done
 if [ -z "$kernel_package" ]
 then
     # kernel package not specified, install a default one
-    # * ubuntu: linux-image-generic
-    # * debian on i386: linux-image-686-pae
-    # * debian on amd64: linux-image-amd64
-    case "$target_arch" in
-    "amd64")
-        kernel_search_regexp="^linux-image-((generic)|(amd64))$"
-        ;;
-    "i386")
-        kernel_search_regexp="^linux-image-((generic)|(686-pae))$"
-        ;;
-    esac
+    kernel_search_regexp="^$(target_kernel_default_package)$"
     error_if_missing="$(
         echo "E: no linux kernel package found."
         echo "E: Run 'debootstick --help-os-support' for more info."
@@ -70,7 +62,7 @@ then
 fi
 
 to_be_installed=""
-for package in $kernel_package_found $OTHER_PACKAGES
+for package in $kernel_package_found $PACKAGES
 do
     if [ $(package_is_installed $package) -eq 0 ]
     then
@@ -94,37 +86,21 @@ rm -rf /var/lib/apt/lists/*
 # tune LVM config
 tune_lvm
 
-if [ "$config_grub_on_serial_line" -gt 0 ]
-then
-    # display the grub interface on serial line
-    cat >> ./etc/default/grub << EOF
-GRUB_TERMINAL=serial
-GRUB_SERIAL_COMMAND="serial --speed=115200 --unit=0 --word=8 --parity=no --stop=1"
-EOF
-fi
-
-# grub conf:
-# * add our custom boot parameters
-# * fix obsolete options in /etc/default/grub
-#   (https://bugs.launchpad.net/ubuntu/+source/grub2/+bug/1258597)
-. /etc/default/grub
-LINUX_OPTIONS="rootdelay=3 $kernel_bootargs"
-GRUB_CMDLINE_LINUX="$GRUB_CMDLINE_LINUX $LINUX_OPTIONS"
-sed -i -e "s/GRUB_CMDLINE_LINUX=.*/GRUB_CMDLINE_LINUX=\"$GRUB_CMDLINE_LINUX\"/" \
-       -e "s/^GRUB_HIDDEN/#GRUB_HIDDEN/g" \
-        /etc/default/grub
-
 # for text console in kvm
 if [ "$debug" = "1" ]
 then
-    # start a shell when the system is ready
-    cat > ./etc/init/ttyS0.conf << EOF
+    # if OS init is upstart, create a service
+    # in order to start a shell when the system is ready
+    if [ -f '/sbin/upstart' ]
+    then
+        cat > ./etc/init/ttyS0.conf << EOF
 start on stopped rc or RUNLEVEL=[12345]
 stop on runlevel [!12345]
 
 respawn
 exec /sbin/getty -L 115200 ttyS0 xterm
 EOF
+    fi
 fi
 
 # set the root password if requested
@@ -141,31 +117,31 @@ case "$root_password_request" in
 esac
 
 echo -n "I: draft image - setting up bootloader... "
-# disable quickboot:
-# work around grub displaying error message with our LVM setup
-# disable vt_handoff:
-# the linux console should be visible during startup (especially
-# if we must enter the root password, or in installer-mode), do
-# not switch to vt7.
-# note: even if the file etc/grub.d/10_linux is re-created
-# after an upgrade of the package grub-common, our script
-# 09_linux_custom will be executed first and take precedence.
-sed -i -e 's/quick_boot=.*/quick_boot=0/' \
-       -e 's/vt_handoff=.*/vt_handoff=0/' etc/grub.d/10_linux
-mv etc/grub.d/10_linux etc/grub.d/09_linux_custom
+target_configure_bootloader
 
-# install grub on this temporary work-image
-# This may not seem useful (it will be repeated on the final
+# installing grub on this temporary work-image
+# may not seem useful (it will be repeated on the final
 # stick anyway), but actually it is:
 # the files created there should be accounted when
 # estimating the final stick minimal size).
-quiet_grub_install $loop_device
+target_install_bootloader
 
-rm boot/grub/device.map
 echo done
 
+# check if target-specific code specified
+# 'applied_kernel_cmdline' variable
+if [ ! -z ${applied_kernel_cmdline+x} ]
+then
+    bootargs="$applied_kernel_cmdline"
+    if [ -z "$bootargs" ]
+    then
+        bootargs="<none>"
+    fi
+    echo "I: draft image - kernel bootargs: $bootargs"
+fi
+
 echo -n "I: draft image - updating fstab... "
-update_fstab $final_lvm_vg
+update_fstab $stick_os_id
 echo done
 
 if [ "$config_hostname" != "" ]
@@ -195,6 +171,7 @@ then
     echo done
 fi
 
+optional_target_cleanup_rootfs draft inside
 # umount all
 undo_all
 
