@@ -2,6 +2,7 @@
 loop_device=$1
 
 eval "$chrooted_functions"
+probe_target_optional_functions
 start_failsafe_mode
 
 # in the chroot commands should use /tmp for temporary files
@@ -12,20 +13,14 @@ export DEBIAN_FRONTEND=noninteractive LANG=C
 failsafe mount -t proc none /proc
 failsafe_mount_sys_and_dev
 
+optional_target_prepare_rootfs final inside
+
 echo -n "I: final image - setting up the bootloader... "
-# let grub find our virtual device
-cd /boot/grub
-cat > device.map << END_MAP
-(hd0) $loop_device
-END_MAP
-
-# install
-quiet_grub_install $loop_device
-
-# remove previous file
-rm /boot/grub/device.map
+target_install_bootloader
 echo done
 
+optional_target_cleanup_rootfs final inside
+
 # umount things
 undo_all
 
